"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
function map(values, transformer, concurrency) {
    return __awaiter(this, void 0, void 0, function* () {
        if (typeof concurrency !== 'number') {
            return Promise.all(values.map(transformer));
        }
        return new Promise((resolve, reject) => {
            let pending = 0;
            let results = [];
            let i = 0;
            let starting = Math.min(concurrency, values.length);
            if (starting <= 0) {
                resolve(results);
                return;
            }
            while (i < starting) {
                next();
            }
            function next() {
                if (i === values.length) {
                    if (!pending) {
                        resolve(results);
                    }
                    return;
                }
                let index = i++;
                let value = values[index];
                pending++;
                let ret;
                try {
                    ret = transformer(value, index, values);
                }
                catch (error) {
                    reject(error);
                    return;
                }
                Promise
                    .resolve(ret)
                    .then(result => {
                    results[index] = result;
                    pending--;
                    next();
                }, error => {
                    reject(error);
                });
            }
        });
    });
}
exports.map = map;
function reduce(values, transformer, initial) {
    return __awaiter(this, void 0, void 0, function* () {
        return values.reduce((result, value, index) => __awaiter(this, void 0, void 0, function* () {
            return transformer(yield result, value, index, values);
        }), initial);
    });
}
exports.reduce = reduce;
function each(values, handler) {
    return __awaiter(this, void 0, void 0, function* () {
        for (let i = 0; i < values.length; i++) {
            if ((yield handler(values[i], i, values)) === false) {
                return false;
            }
        }
        return true;
    });
}
exports.each = each;
function some(values, handler) {
    return __awaiter(this, void 0, void 0, function* () {
        for (let i = 0; i < values.length; i++) {
            if (yield handler(values[i], i, values)) {
                return true;
            }
        }
        return false;
    });
}
exports.some = some;
function every(values, handler) {
    return __awaiter(this, void 0, void 0, function* () {
        for (let i = 0; i < values.length; i++) {
            if (!(yield handler(values[i], i, values))) {
                return false;
            }
        }
        return true;
    });
}
exports.every = every;
//# sourceMappingURL=array.js.map