"use strict";
let awaitableCreators = [];
try {
    const ChildProcessConstructor = require('child_process').ChildProcess;
    awaitableCreators.push((process, ...args) => {
        if (!(process instanceof ChildProcessConstructor) || args.length) {
            return undefined;
        }
        return new Promise((resolve, reject) => {
            process.on('exit', onexit);
            process.on('error', onerror);
            function removeListeners() {
                process.removeListener('exit', onexit);
                process.removeListener('error', onerror);
            }
            function onexit(code) {
                removeListeners();
                if (code === 0) {
                    resolve();
                }
                else {
                    reject(new Error('Invalid exit code'));
                }
            }
            function onerror(error) {
                setImmediate(removeListeners);
                reject(error);
            }
        });
    });
}
catch (error) { }
try {
    const EventEmitterConstructor = require('events').EventEmitter;
    awaitableCreators.push((emitter, types, errorEmitters = []) => {
        if (!(emitter instanceof EventEmitterConstructor)) {
            return undefined;
        }
        errorEmitters.unshift(emitter);
        if (typeof types === 'string') {
            types = [types];
        }
        return new Promise((resolve, reject) => {
            for (let type of types) {
                emitter.on(type, onsuccess);
            }
            for (let emitter of errorEmitters) {
                emitter.on('error', onerror);
            }
            function removeListeners() {
                for (let type of types) {
                    emitter.removeListener(type, onsuccess);
                }
                for (let emitter of errorEmitters) {
                    emitter.removeListener('error', onerror);
                }
            }
            function onsuccess(value) {
                removeListeners();
                resolve(value);
            }
            function onerror(error) {
                setImmediate(removeListeners);
                reject(error);
            }
        });
    });
}
catch (error) { }
function awaitable(target, ...args) {
    for (let creator of awaitableCreators) {
        let promise = creator(target, ...args);
        if (promise) {
            return promise;
        }
    }
    throw new TypeError('Cannot create awaitable from the target object with given arguments');
}
exports.awaitable = awaitable;
//# sourceMappingURL=awaitable.js.map